#!/boot/home/config/bin/perl


# This is a 3-part script for handling playlist queries
# based on MP3 attributes. Different HTML pages are displayed depending
# on whether the user is creating a query, reviewing and saving a query,
# or has just saved a query (confirmation page). The default page comes last.


# Invoke the CGI.pm module with standard CGI functions.
# Guarantees perfect parsing, and much cleaner code
use CGI qw(:standard);

# Invoke the Be::Query module
use Be::SimpleQuery;
use Be::Attribute;

# Suck in the common settings so we can parse for the default query volume
require ('mp3_common.pl');

# get and parse settings using sub defined in mp3_common.pl file
&getsettings;

if ( param() )
{
	my @acceptable_values = ("ListDir","QueryDir","MyIPAddress");
	foreach $value (@acceptable_values)
	{
		# remember to check if value exists - otherwise blank info in config file.
		$prefs{$value} = param("$value") if (param("$value") ne "");
	}

}	

# Initialize a variable so we can get a count of the total number of results in query
$count = 0;

# Set default playlist directory
$ListDir="/boot/home/betips/mp3box/playlists";

# We parse the URL for the presence of these parameters.
# Their presence or absence determines which HTML doc to display
$ThisVol = param("volume");
$SaveList = param("playlist");
$ThisGenre = param("genre");
$ThisTitle = param("title");
$ThisArtist = param("artist");
$ThisAlbum = param("album");
$ThisYear = param("year");

####################
# Show query results if query parameters have been passed
if ($ThisVol) {



# Define some simple HTML variables we can re-use later. Note that lots of HTML
# includes characters that need to be escaped, such as the < and > symbols.
# We avoid having to escape them by surrounding the variable text in quotes.
$highcell="<TD BGCOLOR=#FFFFDA>";
$lowcell="<TD BGCOLOR=#83D1C5>";

# The main contacts page alternates colors for readability. This variable
# will be toggled between the high and low colors after each record.
# We'll start with the low value.
$cellopen=$lowcell;



print <<EOF;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>Query Results</title>
</head>
<body bgcolor="#ffffff">
<blockquote>

<h2>Query Results</h2>
<center>
<table border=1 cellpadding=5>
	<tr>
		<td>
		Volume:  $ThisVol <br>
		Genre:  $ThisGenre <br>
		Artist:  $ThisArtist <br>
		Title contains:  $ThisTitle <br>
		Year is:  $ThisYear <br>
		</td>

		<td>
		<FORM ACTION="/cgi-bin/mp3_query.pl">
		<INPUT TYPE="submit" VALUE="Save this playlist as ..."><INPUT NAME="playlist" VALUE="$ThisGenre">
		</FORM>
<P>
		<FORM ACTION="/cgi-bin/mp3_query.pl">
		<INPUT TYPE="submit" VALUE="New query">
		</FORM>
		
		<FORM ACTION="/cgi-bin/mp3_base.pl">
		<INPUT TYPE="submit" VALUE="Return to base">
		</FORM>
				


		</td>

	</tr>
</table>
</center>
<P>
<center>
<TABLE WIDTH=95% BORDER=0 CELLPADDING=5 CELLSPACING=0>

	<TD BGCOLOR=#46528A><FONT COLOR=#D0D7BB SIZE=+1><B>Title</b></FONT></TD>
	<TD BGCOLOR=#46528A><FONT COLOR=#D0D7BB SIZE=+1><B>Artist</b></FONT></TD>
	<TD BGCOLOR=#46528A><FONT COLOR=#D0D7BB SIZE=+1><B>Genre</b></FONT></TD>
EOF


# Here's the meat -- the actual query is printed both to the page
# and to a temp file. If the user saves the playlist, the temp file
# is moved to a permanent location

# Open a temp file to write to
open (QRESULTS, ">/tmp/query_results") || die "cannot create output file\n";

# Creates an array with the results from a query

# This method (qq) handles the escaping for you. 
# Otherwise you just have to escape all the double quotes

@files = Be::SimpleQuery::SimpleQuery("$ThisVol", "((Audio:Genre==\"*$ThisGenre*\")&&(Audio:Artist==\"*$ThisArtist*\")&&(Audio:Title==\"*$ThisTitle*\")&&(Audio:Album==\"*$ThisAlbum*\")&&(BEOS:TYPE=='audio/x-mpeg'))");

# Here's some leftover syntax for the standard Be::Query module
#@files = Be::Query::Query($ThisVol,
#          qq[((Audio:Genre=="*$ThisGenre*")&&(Audio:Artist=="*$ThisArtist*")&&(Audio:Title=="*$ThisTitle*")&&(Audio:Album=="*$ThisAlbum*")&&(Audio:Year=="*$ThisYear*")&&(BEOS:TYPE=="audio/x-mpeg"))]
#        );
        

        
foreach(@files) {
#	print "$_<BR>\n";
# We print the query results both to the screen and to a temp file. 
# First to the screen (the fancy stuff)

	$MP3Node = Be::Attribute::GetBNode("$_");
	$Artist = Be::Attribute::ReadAttr($MP3Node, "Audio:Artist");
	$Title = Be::Attribute::ReadAttr($MP3Node, "Audio:Title");
	$Genre = Be::Attribute::ReadAttr($MP3Node, "Audio:Genre");
	$Icon = "/mp3box/gfx/swirl.gif border=0 width=23 height=21";	
	Be::Attribute::CloseNode ($MP3Node);

# Create a link form with spaces replaced by %20 so we can listen to playlist results
			$Link = $_;
			$Link =~ s/\ /%20/g;
			
print <<EOF;
	
		<TR VALIGN=BOTTOM>


			$cellopen$Title</TD>

			$cellopen$Artist</TD>

			$cellopen$Genre</TD>


		</TR>

EOF


		# To get alternating stripes on the index page, we Toggle the value of the 
		# cell highlighting variable back and forth, one toggle for each record.
		if ($cellopen eq $highcell) {
			$cellopen=$lowcell;
			}
		else {
			$cellopen=$highcell;
			}

# Now just print the line to the temp file, which may become a playlist if user saves it later
	print QRESULTS "$_\n";
	$count = $count + 1;
	}
print "</TABLE>	\n";
print "</CENTER>	\n";
close (QRESULTS);

# Print total number of results
print "<P>\n";
print "<b>Total matches: $count</b>\n";


print <<EOF;

<P>



</blockquote>
</body>
</html>


EOF
} 
####################
# Show query save page if save parameter is present

elsif ($SaveList) {


# The user chose to save the file on the previous page, so now we
# move the temp file to the playlists dir, and set its mime type properly.
system qq[mv /tmp/query_results "$ListDir/$SaveList"];
system qq[settype -t text/x-soundplay-playlist "$ListDir/$SaveList"];

print <<EOF;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>Playlist saved</title>
</head>
<body bgcolor="#FFFFFF">
<blockquote>

<h2>Playlist saved as \"$SaveList\"</h2>
EOF

# This lets us link to lists with spaces in their names, while
# not having to insert %20s in the visible link
$ListName = $SaveList;
$SaveList =~ s/\ /%20/g;

print <<EOF;
<h3>Play <a href="/cgi-bin/mp3_list.pl?List=$SaveList">$ListName</a></h3>

<P>
		<FORM ACTION="/cgi-bin/mp3_query.pl">
		<INPUT TYPE="submit" VALUE="New query">
		</FORM>
		
		<FORM ACTION="/cgi-bin/mp3_base.pl">
		<INPUT TYPE="submit" VALUE="Return to base">
		</FORM>

</blockquote>
</body>
</html>


EOF
}
####################
# Show default page if no parameters are present
else {

print <<EOF;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
	<title>Query-based playlist generator</title>
</head>
<body bgcolor="#FFFFFF">
<blockquote>

<h2>Query-based playlist generator</h2>
<i>All strings are case-sensitive!</i>
<P>
<FORM ACTION="/cgi-bin/mp3_query.pl">
Volume name: <INPUT NAME="volume" VALUE="$prefs{'QueryDir'}">
<P>
Song title contains: <INPUT NAME="title" VALUE="">
<P>
Artist name contains: <INPUT NAME="artist" VALUE="">
<P>
Album name contains: <INPUT NAME="album" VALUE="">
<P>
Year recorded: <INPUT NAME="year" VALUE="">
<P>

Genre: 
<SELECT NAME="genre">

<OPTION VALUE="">Any
<OPTION VALUE="Blues">Blues
<OPTION VALUE="Classic Rock">Classic Rock
<OPTION VALUE="Country">Country
<OPTION VALUE="Dance">Dance
<OPTION VALUE="Disco">Disco
<OPTION VALUE="Funk">Funk
<OPTION VALUE="Grunge">Grunge
<OPTION VALUE="Hip-Hop">Hip-Hop
<OPTION VALUE="Jazz">Jazz
<OPTION VALUE="Metal">Metal
<OPTION VALUE="New Age">New Age
<OPTION VALUE="Oldies">Oldies
<OPTION VALUE="Other">Other
<OPTION VALUE="Pop">Pop
<OPTION VALUE="R&B">R&B
<OPTION VALUE="Rap">Rap
<OPTION VALUE="Reggae">Reggae
<OPTION VALUE="Rock">Rock
<OPTION VALUE="Techno">Techno
<OPTION VALUE="Industrial">Industrial
<OPTION VALUE="Alternative">Alternative
<OPTION VALUE="Ska">Ska
<OPTION VALUE="Death Metal">Death Metal
<OPTION VALUE="Pranks">Pranks
<OPTION VALUE="Soundtrack">Soundtrack
<OPTION VALUE="Euro-Techno">Euro-Techno
<OPTION VALUE="Ambient">Ambient
<OPTION VALUE="Trip-Hop">Trip-Hop
<OPTION VALUE="Vocal">Vocal
<OPTION VALUE="Jazz+Funk">Jazz+Funk
<OPTION VALUE="Fusion">Fusion
<OPTION VALUE="Trance">Trance
<OPTION VALUE="Classical">Classical
<OPTION VALUE="Instrumental">Instrumental
<OPTION VALUE="Acid">Acid
<OPTION VALUE="House">House
<OPTION VALUE="Game">Game
<OPTION VALUE="Sound Clip">Sound Clip
<OPTION VALUE="Gospel">Gospel
<OPTION VALUE="Noise">Noise
<OPTION VALUE="Alt. Rock">Alt. Rock
<OPTION VALUE="Bass">Bass
<OPTION VALUE="Soul">Soul
<OPTION VALUE="Punk">Punk
<OPTION VALUE="Space">Space
<OPTION VALUE="Meditative">Meditative
<OPTION VALUE="Instrum. Pop">Instrum. Pop
<OPTION VALUE="Instrum. Rock">Instrum. Rock
<OPTION VALUE="Ethnic">Ethnic
<OPTION VALUE="Gothic">Gothic
<OPTION VALUE="Darkwave">Darkwave
<OPTION VALUE="Techno-Indust.">Techno-Indust.
<OPTION VALUE="Electronic">Electronic
<OPTION VALUE="Pop-Folk">Pop-Folk
<OPTION VALUE="Eurodance">Eurodance
<OPTION VALUE="Dream">Dream
<OPTION VALUE="Southern Rock">Southern Rock
<OPTION VALUE="Comedy">Comedy
<OPTION VALUE="Cult">Cult
<OPTION VALUE="Gangsta">Gangsta
<OPTION VALUE="Top 40">Top 40
<OPTION VALUE="Christian Rap">Christian Rap
<OPTION VALUE="Pop/Funk">Pop/Funk
<OPTION VALUE="Jungle">Jungle
<OPTION VALUE="Native American">Native American
<OPTION VALUE="Cabaret">Cabaret
<OPTION VALUE="New Wave">New Wave
<OPTION VALUE="Psychadelic">Psychadelic
<OPTION VALUE="Rave">Rave
<OPTION VALUE="Showtunes">Showtunes
<OPTION VALUE="Trailer">Trailer
<OPTION VALUE="Lo-Fi">Lo-Fi
<OPTION VALUE="Tribal">Tribal
<OPTION VALUE="Acid Punk">Acid Punk
<OPTION VALUE="Acid Jazz">Acid Jazz
<OPTION VALUE="Polka">Polka
<OPTION VALUE="Retro">Retro
<OPTION VALUE="Musical">Musical
<OPTION VALUE="Rock & Roll">Rock & Roll
<OPTION VALUE="Hard Rock">Hard Rock
<OPTION VALUE="Folk">Folk
<OPTION VALUE="Folk/Rock">Folk/Rock
<OPTION VALUE="National Folk">National Folk
<OPTION VALUE="Swing">Swing
<OPTION VALUE="Fusion">Fusion
<OPTION VALUE="Bebob">Bebob
<OPTION VALUE="Latin">Latin
<OPTION VALUE="Revival">Revival
<OPTION VALUE="Celtic">Celtic
<OPTION VALUE="Bluegrass">Bluegrass
<OPTION VALUE="Avantgarde">Avantgarde
<OPTION VALUE="Gothic Rock">Gothic Rock
<OPTION VALUE="Progress. Rock">Progress. Rock
<OPTION VALUE="Psychadel. Rock">Psychadel. Rock
<OPTION VALUE="Symphonic Rock">Symphonic Rock
<OPTION VALUE="Slow Rock">Slow Rock
<OPTION VALUE="Big Band">Big Band
<OPTION VALUE="Chorus">Chorus
<OPTION VALUE="Easy Listening">Easy Listening
<OPTION VALUE="Acoustic">Acoustic
<OPTION VALUE="Speech">Speech
<OPTION VALUE="Chanson">Chanson
<OPTION VALUE="Opera">Opera
<OPTION VALUE="Chamber Music">Chamber Music
<OPTION VALUE="Sonata">Sonata
<OPTION VALUE="Symphony">Symphony
<OPTION VALUE="Booty Bass">Booty Bass
<OPTION VALUE="Primus">Primus
<OPTION VALUE="Porn Groove">Porn Groove
<OPTION VALUE="Satire">Satire
			
</SELECT>
<P>
<INPUT TYPE="submit" VALUE="Look for songs">
</FORM>

<P>

<FORM ACTION="/cgi-bin/mp3_base.pl">
<INPUT TYPE="submit" VALUE="Return to base">
</FORM>
		
</blockquote>
</body>
</html>


EOF
}

